@@ -19,7 +19,7 @@ pub type MultiFuze<T> = MultiFuse<T>;
 /// Like one-time use `Bomb`s, `MultiBomb`s can be safely cloned and sent between threads.
 /// 
 /// [`exploded()`]: ../struct.Bomb.html#method.exploded
-pub struct MultiBomb<T> {
+pub struct MultiBomb<T: Clone> {
     inner: Bomb<(MultiBomb<T>, T)>
 }
 
@@ -49,12 +49,12 @@ pub struct MultiBomb<T> {
 /// 
 /// You could even use a one-time use [`Bomb<()>`](Bomb) as a drop signal if you wanted to, though it 
 /// may be redundant since [`MultiBomb`]s already use [`Bomb`]s internally.
-pub struct MultiFuse<T> {
+pub struct MultiFuse<T: Clone> {
     // TODO: Implement proper drop behaviour, read above for further info. 
     inner: Fuse<(MultiBomb<T>, T)>
 }
 
-impl<T> MultiBomb<T> {
+impl<T: Clone> MultiBomb<T> {
     /// Creates a new single but reusable producer [`MultiFuse<T>`], and a mutli-consumer `MultiBomb<T>`.
     /// 
     /// Instances of `MultiBomb<T>` may be safely cloned and sent between threads.
@@ -89,25 +89,22 @@ impl<T> MultiBomb<T> {
     /// 
     /// See [`MultiFuse`] for more details.
     pub fn exploded(&mut self) -> Option<T> {
-        match self.inner.exploded() {
-            None => None,
+        let (new_bomb, value) = match self.inner.exploded() {
+            None => return None,
 
-            Some((new_bomb, value)) => {
-                // Force a clone to increment strong counter.
-                self.inner = new_bomb.inner.clone();
+            // Clone MultiBomb to increment strong counter.
+            // Clone data value to properly construct/deconstruct
+            // on this thread.
+            Some(data) => data.clone()
+        };
+        
+        self.inner = new_bomb.inner;
 
-                // Forget read value, as it would have been
-                // copied, and therefore would break the reference
-                // counter if allowed to drop.
-                std::mem::forget(new_bomb);
-
-                Some(value)
-            }
-        }
+        Some(value)
     }
 }
 
-impl<T> MultiFuse<T> {
+impl<T: Clone> MultiFuse<T> {
     fn new(_counter: &Arc<()>) -> Self {
         Self {
             inner: Fuse::new(_counter),
@@ -140,7 +137,7 @@ impl<T> MultiFuse<T> {
     }
 }
 
-impl<T> Clone for MultiBomb<T> {
+impl<T: Clone> Clone for MultiBomb<T> {
     fn clone(&self) -> Self {
         Self { inner: self.inner.clone() }
     }
